@@ -17,10 +17,10 @@ from prompt_toolkit.contrib.regular_languages.completion import GrammarCompleter
 from prompt_toolkit.contrib.regular_languages.lexer import GrammarLexer
 from prompt_toolkit.document import Document
 from prompt_toolkit.interface import CommandLineInterface
-from prompt_toolkit.layout.controls import TokenListControl
-from prompt_toolkit.shortcuts import create_eventloop
+from prompt_toolkit.layout.lexers import PygmentsLexer, SimpleLexer
 
-from ptpython.python_input import PythonInput, PythonValidator, PythonCompleter
+from .python_input import PythonInput, PythonValidator, PythonCompleter
+from .eventloop import create_eventloop
 
 from IPython.terminal.embed import InteractiveShellEmbed as _InteractiveShellEmbed
 from IPython.terminal.ipapp import load_default_config
@@ -29,22 +29,34 @@ from IPython.core.inputsplitter import IPythonInputSplitter
 
 from pygments.lexers import PythonLexer, BashLexer
 from pygments.token import Token
+from ptpython.prompt_style import PromptStyle
 
 __all__ = (
     'embed',
 )
 
 
-class IPythonPrompt(TokenListControl):
+class IPythonPrompt(PromptStyle):
     """
-    Prompt showing something like "In [1]:".
+    PromptStyle that uses the templates, as set by IPython.
+    Usually, something like "In [1]:".
     """
     def __init__(self, prompt_manager):
-        def get_tokens(cli):
-            text = prompt_manager.render('in', color=False, just=False)
-            return [(Token.In, text)]
+        self.prompt_manager = prompt_manager
 
-        super(IPythonPrompt, self).__init__(get_tokens)
+    def in_tokens(self, cli):
+        text = self.prompt_manager.render('in', color=False, just=False)
+        return [(Token.In, text)]
+
+    def in2_tokens(self, cli, width):
+        text = self.prompt_manager.render('in2', color=False, just=False)
+        return [(Token.In, text.rjust(width))]
+
+    def out_tokens(self, cli):
+        # This function is currently not used by IPython. But for completeness,
+        # it would look like this.
+        text = self.prompt_manager.render('out', color=False, just=False)
+        return [(Token.Out, text)]
 
 
 class IPythonValidator(PythonValidator):
@@ -104,14 +116,12 @@ def create_lexer():
 
     return GrammarLexer(
         g,
-        tokens={
-            'percent': Token.Operator,
-            'magic': Token.Keyword,
-            'filename': Token.Name,
-        },
         lexers={
-            'python': PythonLexer,
-            'system': BashLexer,
+            'percent': SimpleLexer(Token.Operator),
+            'magic': SimpleLexer(Token.Keyword),
+            'filename': SimpleLexer(Token.Name),
+            'python': PygmentsLexer(PythonLexer),
+            'system': PygmentsLexer(BashLexer),
         })
 
 
@@ -151,12 +161,15 @@ class IPythonInput(PythonInput):
                                             ipython_shell.magics_manager,
                                             ipython_shell.alias_manager)
         kw['_lexer'] = create_lexer()
-        kw['_validator'] = IPythonValidator()
-        kw['_python_prompt_control'] = IPythonPrompt(ipython_shell.prompt_manager)
+        kw['_validator'] = IPythonValidator(
+            get_compiler_flags=self.get_compiler_flags)
 
         super(IPythonInput, self).__init__(*a, **kw)
         self.ipython_shell = ipython_shell
 
+        self.all_prompt_styles['ipython'] = IPythonPrompt(ipython_shell.prompt_manager)
+        self.prompt_style = 'ipython'
+
 
 class InteractiveShellEmbed(_InteractiveShellEmbed):
     """
@@ -169,6 +182,7 @@ class InteractiveShellEmbed(_InteractiveShellEmbed):
         vi_mode = kw.pop('vi_mode', False)
         history_filename = kw.pop('history_filename', None)
         configure = kw.pop('configure', None)
+        title = kw.pop('title', None)
 
         super(InteractiveShellEmbed, self).__init__(*a, **kw)
 
@@ -181,6 +195,9 @@ class InteractiveShellEmbed(_InteractiveShellEmbed):
             get_globals=get_globals, vi_mode=vi_mode,
             history_filename=history_filename)
 
+        if title:
+            ipython_input.terminal_title = title
+
         if configure:
             configure(ipython_input)
 
@@ -191,7 +208,7 @@ class InteractiveShellEmbed(_InteractiveShellEmbed):
     def raw_input(self, prompt=''):
         print('')
         try:
-            string = self._cli.run().text
+            string = self._cli.run(reset_current_buffer=True).text
 
             # In case of multiline input, make sure to append a newline to the input,
             # otherwise, IPython will ask again for more input in some cases.
