@@ -14,11 +14,11 @@ from pygments.styles.default import DefaultStyle
 
 from prompt_toolkit.application import AbortAction
 from prompt_toolkit.enums import DEFAULT_BUFFER
-from prompt_toolkit.interface import AcceptAction, CommandLineInterface
+from prompt_toolkit.interface import AcceptAction
 from prompt_toolkit.layout.utils import token_list_width
 from prompt_toolkit.shortcuts import create_asyncio_eventloop
-from prompt_toolkit.styles import PygmentsStyle
-from prompt_toolkit.utils import DummyContext, Callback
+from prompt_toolkit.styles import style_from_pygments
+from prompt_toolkit.utils import DummyContext
 
 from .python_input import PythonInput, PythonCommandLineInterface
 from .eventloop import create_eventloop
@@ -44,7 +44,7 @@ class PythonRepl(PythonInput):
         kw.update({
             '_accept_action': AcceptAction.run_in_terminal(
                 handler=self._process_document, render_cli_done=True),
-            '_on_start': Callback(self._on_start),
+            '_on_start': self._on_start,
             '_on_exit': AbortAction.RETURN_NONE,
         })
 
@@ -61,7 +61,7 @@ class PythonRepl(PythonInput):
                         code = compile(f.read(), path, 'exec')
                         six.exec_(code, self.get_globals(), self.get_locals())
                 else:
-                    output = self.cli.output
+                    output = cli.output
                     output.write('WARNING | File not found: {}\n\n'.format(path))
 
     def _process_document(self, cli, buffer):
@@ -76,6 +76,9 @@ class PythonRepl(PythonInput):
             except Exception as e:
                 self._handle_exception(cli, e)
 
+            if self.insert_blank_line_after_output:
+                cli.output.write('\n')
+
             self.current_statement_index += 1
             self.signatures = []
 
@@ -136,7 +139,6 @@ class PythonRepl(PythonInput):
                 code = compile_with_flags(line, 'exec')
                 six.exec_(code, self.get_globals(), self.get_locals())
 
-            output.write('\n')
             output.flush()
 
     @classmethod
@@ -163,9 +165,9 @@ class PythonRepl(PythonInput):
         # (We use the default style. Most other styles result
         # in unreadable colors for the traceback.)
         tokens = _lex_python_traceback(tb)
-        cli.print_tokens(tokens, style=PygmentsStyle(DefaultStyle))
+        cli.print_tokens(tokens, style=style_from_pygments(DefaultStyle))
 
-        output.write('%s\n\n' % e)
+        output.write('%s\n' % e)
         output.flush()
 
     @classmethod
